#!/usr/bin/python
# -*- coding: utf-8 -*-

# SAT plugin for managing xep-0049
# Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Jérôme Poisson (goffi@goffi.org)

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from sat.core.i18n import _
from logging import debug, info, error, warning
from twisted.internet import defer
from wokkel import compat
from twisted.words.xish import domish



PLUGIN_INFO = {
    "name": "XEP-0049 Plugin",
    "import_name": "XEP-0049",
    "type": "XEP",
    "protocols": ["XEP-0049"],
    "dependencies": [],
    "main": "XEP_0049",
    "handler": "no",
    "description": _("""Implementation of private XML storage""")
}


class XEP_0049(object):
    NS_PRIVATE = 'jabber:iq:private'

    def __init__(self, host):
        info(_("Plugin XEP-0049 initialization"))
        self.host = host

    @defer.inlineCallbacks
    def privateXMLStore(self, element, profile_key):
        """Store private data
        @param element: domish.Element to store (must have a namespace)
        @param profile_key: %(doc_profile_key)s

        """
        assert isinstance(element, domish.Element)
        client = self.host.getClient(profile_key)
        yield self.host.checkFeature(XEP_0049.NS_PRIVATE, profile_key=client.profile)
        iq_elt = compat.IQ(client.xmlstream)
        query_elt = iq_elt.addElement('query', XEP_0049.NS_PRIVATE)
        query_elt.addChild(element)
        yield iq_elt.send()

    @defer.inlineCallbacks
    def privateXMLGet(self, node_name, namespace, profile_key):
        """Store private data
        @param node_name: name of the node to get
        @param namespace: namespace of the node to get
        @param profile_key: %(doc_profile_key)s
        @return (domish.Element): a deferred which fire the stored data

        """
        client = self.host.getClient(profile_key)
        yield self.host.checkFeature(XEP_0049.NS_PRIVATE, profile_key=client.profile)
        iq_elt = compat.IQ(client.xmlstream, 'get')
        query_elt = iq_elt.addElement('query', XEP_0049.NS_PRIVATE)
        query_elt.addElement(node_name, namespace)
        answer_iq_elt = yield iq_elt.send()
        answer_query_elt = answer_iq_elt.elements(XEP_0049.NS_PRIVATE, 'query').next()
        defer.returnValue(answer_query_elt.firstChildElement())

